@@ -1,4 +1,3 @@
-import sys
 import copy
 from typing import Union
 from collections import defaultdict
@@ -109,7 +108,6 @@ class Convert:
         result = defaultdict(list)
         for insert_unit in extracted_data:
             collection = list(insert_unit.keys())[0]
-
             for doc in insert_unit[collection]:
                 result[collection].append(doc)
 
@@ -127,28 +125,24 @@ class Convert:
         :rtype: dict
         """
 
-        def child_delete(doc_with_child: dict) -> None:
+        def child_delete(doc: dict) -> None:
             """
             子要素を削除する
 
-            :param dict doc_with_child:
+            :param dict doc:
             :return:
             """
-            tmp = copy.deepcopy(doc_with_child)
-            tmp_list = []
+            tmp = copy.deepcopy(doc)
 
             # 子要素のデータを抽出
-            for k, v in doc_with_child.items():
-                if self.parent != k and self.child != k and (
+            key_list = [k for k, v in doc.items()
+                        if self.parent != k and self.child != k and (
                         isinstance(v, dict) or (
                         isinstance(v, list) and (
-                        not Utils.item_literal_check(v)))):
-                    tmp_list.append(k)
+                        not Utils.item_literal_check(v))))]
 
             # 該当データがtmpにあれば削除
-            for j in tmp_list:
-                if j in tmp:
-                    del tmp[j]
+            tmp = {k: v for k, v in tmp.items() if k not in key_list}
 
             # outputのデータを入れ替える
             if collection in output:
@@ -252,7 +246,7 @@ class Convert:
                                 self.parent: ref_list[parent]
                             })
 
-                        # 子要素だけのデータの場合else区へ飛ばないのでここでoidを作成する
+                        # 子要素だけのデータの場合elseへ飛ばないのでここでoidを作成する
                         if '_id' not in tmp:
                             tmp.update({'_id': ref_list[my].id})
 
@@ -305,18 +299,18 @@ class Convert:
                 if isinstance(value, dict):
                     if key == pull_key:
                         output.update({key: value})
+                        break
                     recursive(value)
                 elif isinstance(value, list) and Utils.item_literal_check(
                         value):
-                    pass
+                    continue
                 elif isinstance(value, list):
                     if key == pull_key:
                         output.update({key: value})
-                    tmp_list = []
-                    for i in value:
-                        tmp_list.append(recursive(i))
+                        break
+                    [recursive(i) for i in value]
                 else:
-                    pass
+                    continue
 
         output = {}
         recursive(data)
@@ -329,7 +323,7 @@ class Convert:
 
         :param dict data:
         :param tuple ex_keys:
-        :return: result
+        :return:
         :rtype: dict
         """
 
@@ -339,20 +333,20 @@ class Convert:
                 if isinstance(value, dict):
                     if key in ex_keys:
                         continue
-                    output.update({key: recursive(value)})
+                    o = {key: recursive(value)}
                 elif isinstance(value, list) and Utils.item_literal_check(
                         value):
-                    output.update({key: value})
+                    o = {key: value}
                 elif isinstance(value, list):
                     if key in ex_keys:
                         continue
-                    output.update({key: [recursive(i) for i in value]})
+                    o = {key: [recursive(i) for i in value]}
                 else:
-                    output.update({key: value})
+                    o = {key: value}
+                output.update(o)
             return output
 
-        result = recursive(data)
-        return result
+        return recursive(data)
 
     @staticmethod
     def _attached_oid(data: dict) -> dict:
@@ -371,9 +365,8 @@ class Convert:
                 else:
                     for doc in v:
                         doc.update({'_id': ObjectId()})
-
-            except ValueError as e:
-                sys.exit(e)
+            except ValueError:
+                raise
         return data
 
     def _date_replace(self, list_data: list) -> list:
@@ -415,40 +408,33 @@ class Convert:
             :rtype: dict
             """
             output = {}
-
             for key, value in data.items():
-
                 if isinstance(value, dict):
                     if not Utils.collection_name_check(key):
                         raise EdmanFormatError(f'この名前は使用できません {key}')
-
                     converted_value = self._convert_datetime(value)
-                    output.update({key: recursive(converted_value)})
+                    o = {key: recursive(converted_value)}
 
                 elif isinstance(value, list):
                     # 日付データが含まれていたらdatetimeオブジェクトに変換
                     value = self._date_replace(value)
-
                     # 通常のリストデータの場合
                     if Utils.item_literal_check(value):
                         if not Utils.field_name_check(key):
                             raise EdmanFormatError(f'フィールド名に不備があります {key}')
-
                         list_tmp_data = value
                     # 子要素としてのリストデータの場合
                     else:
                         if not Utils.collection_name_check(key):
                             raise EdmanFormatError(f'この名前は使用できません {key}')
-
                         list_tmp_data = [recursive(self._convert_datetime(i))
                                          for i in value]
-                    output.update({key: list_tmp_data})
-
+                    o = {key: list_tmp_data}
                 else:
                     if not Utils.field_name_check(key):
                         raise EdmanFormatError(f'フィールド名に不備があります {key}')
-
-                    output.update({key: value})
+                    o = {key: value}
+                output.update(o)
             return output
 
         result = recursive(raw_data)
