@@ -1,10 +1,9 @@
-import sys
 import copy
+import urllib.parse
 from typing import Union
-import jmespath
+from jmespath import exceptions as jms_exceptions, search as jms_search
 from pymongo import MongoClient, errors
 from bson import ObjectId, DBRef
-from tqdm import tqdm
 from edman.utils import Utils
 from edman.exceptions import (EdmanDbConnectError, EdmanInternalError,
                               EdmanDbProcessError, EdmanFormatError)
@@ -21,10 +20,9 @@ class DB:
 
         if con is not None:
             try:
-                self.db = self._connect(**con)
+                self.db, self.client = self._connect(**con)
+
             except EdmanDbConnectError:
-                # TODO ロギングで保存する traceback.format_exc()
-                # TODO raiseを入れると動かない？
                 raise
             except Exception:
                 raise
@@ -51,21 +49,46 @@ class DB:
         """
         | DBに接続
         | self.edman_dbというメンバ変数には、このメソッドでDBオブジェクトが入る
+        |
+        | auth enabled example
+        | kwargs = {
+        |   'user':'user_name',
+        |   'host':'127.0.0.1',
+        |   'port':'27017',
+        |   'database':'db_name',
+        |   'password':'password',
+        |   'options':['authSource=db_name']
+        | }
+        | auth enabled and LDAP connection example
+        | kwargs = {
+        |   'user':'ldap_user_name',
+        |   'host':'127.0.0.1',
+        |   'port':'27017',
+        |   'database':'db_name',
+        |   'password':'password',
+        |   'options':['authMechanism=PLAIN']
+        | }
 
         :param dict kwargs: DB接続情報の辞書
         :return: DB接続インスタンス(self.edman_db)
         """
         host = kwargs['host']
         port = kwargs['port']
-        database = kwargs['database']
-        auth_database = kwargs['auth_database']
-        user = kwargs['user']
-        password = kwargs['password']
-        statement = f'mongodb://{user}:{password}@{host}:{port}/{auth_database}'
-        client = MongoClient(statement)
+        database = str(kwargs['database'])
+        user = urllib.parse.quote_plus(str(kwargs['user']))
+        password = urllib.parse.quote_plus(str(kwargs['password']))
+
+        statement = ""
+        if kwargs.get('options'):
+            for option in kwargs['options']:
+                connector = '&' if len(statement) else '?'
+                statement += connector + option
+
+        mongo_uri = f'mongodb://{user}:{password}@{host}:{port}/{statement}'
+        client = MongoClient(mongo_uri)
 
         try:  # サーバの接続確認
-            client.admin.command('ismaster')
+            client.admin.command('ping')
         except errors.OperationFailure:
             raise EdmanDbConnectError('Invalid account.')
         except errors.ConnectionFailure:
@@ -75,71 +98,212 @@ class DB:
 
         edman_db = client[database]
 
-        try:  # 現状、認証を確認する方法がないため、これで代用
+        try:  # 現状、承認を確認する方法がないため、これで代用
             edman_db.list_collection_names()
         except errors.OperationFailure:
             raise EdmanDbConnectError('Authentication failed.')
         except Exception as e:
             raise EdmanDbConnectError(e)
 
-        return edman_db
+        return edman_db, client
 
-    def insert(self, insert_data: list, cmd=True) -> list:
+    def create_user_and_role(self, db_name: str, user_name: str, pwd: str,
+                             role='readWrite', role_name='edman') -> None:
+        """
+        指定のロールとユーザを作成
+        DB内部ユーザのみ対象
+        DB内部ユーザを作成するメソッドなので、各ユーザのDBにロールとユーザ情報を作成する
+        admin権限のみ使える
+
+        :param str db_name:
+        :param str user_name:
+        :param str pwd:
+        :param str role: default 'readWrite'
+        :param str role_name: default 'edman'
+        :return:
+        """
+        # ロール作成
+        try:
+            self.create_role_for_dbuser(db_name, role_name, role)
+        except EdmanDbProcessError:
+            raise
+
+        # ユーザ作成とロール適応
+        try:
+            self.client[db_name].command(
+                "createUser",
+                user_name,
+                pwd=pwd,
+                roles=[role_name]
+            )
+        except AttributeError:
+            raise EdmanDbProcessError('接続処理されていません')
+        except (errors.OperationFailure, errors.InvalidName):
+            self.client[db_name].command('dropRole', role_name)
+            raise EdmanDbProcessError('ユーザの作成処理でエラーが起きました')
+        except:
+            raise
+
+    def create_role_for_dbuser(self, db_name: str, role_name: str,
+                               role='readWrite') -> None:
+        """
+        ロールを作成
+
+        admin権限のみ使える
+        ユーザDBにロールを作成
+
+        :param str db_name:
+        :param str role_name:
+        :param str role: default 'readWrite'
+        :return:
+        """
+        try:
+            self.client[db_name].command(
+                "createRole",
+                role_name,
+                privileges=[
+                    {
+                        "resource": {"db": db_name, "collection": ""},
+                        "actions": ["changeOwnPassword"]
+                    }
+                ],
+                roles=[
+                    {
+                        'role': role,
+                        'db': db_name,
+                    },
+                ],
+            )
+        except AttributeError:
+            raise EdmanDbProcessError('接続処理されていません')
+        except (errors.OperationFailure, errors.InvalidName):
+            raise EdmanDbProcessError('ロール作成処理でエラーが起きました')
+        except:
+            raise
+
+    def create_role(self, db_name: str, role_name: str,
+                    role='readWrite') -> None:
+        """
+        ロールを作成
+
+        admin権限のみ使える
+        LDAPユーザ用　admin DBにロールを作成、role_nameはグループのdn
+
+        :param str db_name:
+        :param str role_name:
+        :param str role: default 'readWrite'
+        :return:
+        """
+        try:
+            self.db.command(
+                "createRole",
+                role_name,
+                privileges=[
+                    {
+                        "resource": {"db": db_name, "collection": ""},
+                        "actions": ["changeOwnPassword"]
+                    }
+                ],
+                roles=[
+                    {
+                        'role': role,
+                        'db': db_name,
+                    },
+                ],
+            )
+        except AttributeError:
+            raise EdmanDbProcessError('接続処理されていません')
+        except (errors.OperationFailure, errors.InvalidName):
+            raise EdmanDbProcessError('ロール作成処理でエラーが起きました')
+        except:
+            raise
+
+    def delete_user_and_role(self, user_name: str, db_name: str,
+                             role_name='edman', admin_name='admin') -> None:
+        """
+        ユーザ削除
+        adminのみ実行可能
+
+        :param str user_name:
+        :param str db_name:
+        :param str role_name: default 'edman'
+        :param str admin_name: default 'admin'
+        :return:
+        """
+        if user_name == admin_name:
+            raise EdmanDbProcessError(f"{admin_name}は管理者なので削除できません")
+        try:
+            cl = self.client
+            cl[db_name].command("dropUser", user_name)
+            self.delete_role(role_name, db_name)
+        except AttributeError:
+            raise EdmanDbProcessError('接続処理されていません')
+        except (errors.OperationFailure, errors.InvalidName):
+            raise EdmanDbProcessError('ユーザ及びロールの削除処理でエラーが起きました')
+        except:
+            raise
+
+    def delete_db(self, delete_db_name: str, admin_db='admin') -> None:
+        """
+        DBの削除
+
+        :param str delete_db_name:
+        :param str admin_db:
+        :return:
+        """
+        if delete_db_name == admin_db:
+            raise EdmanDbProcessError('管理者DBは削除できません')
+        try:
+            cl = self.client
+            if delete_db_name in cl.list_database_names():
+                cl.drop_database(delete_db_name)
+        except AttributeError:
+            raise EdmanDbProcessError('接続処理されていません')
+        except errors.OperationFailure:
+            raise EdmanDbProcessError('DBの削除処理でエラーが起きました')
+        except:
+            raise
+
+    def delete_role(self, role_name: str, target_db: str) -> None:
+        """
+        指定されたDB内のロールの削除
+        admin権限のみ
+
+        :param str role_name:
+        :param str target_db:
+        :return:
+        """
+        try:
+            cl = self.client
+            cl[target_db].command("dropRole", role_name)
+        except AttributeError:
+            raise EdmanDbProcessError('接続処理されていません')
+        except errors.InvalidName:
+            raise EdmanDbProcessError('ロール名が不正です')
+        except errors.OperationFailure:
+            raise EdmanDbProcessError('ロール削除処理でエラーが起きました')
+        except:
+            raise
+
+    def insert(self, insert_data: list) -> list:
         """
         インサート実行
 
         :param list insert_data: バルクインサート対応のリストデータ
-        :param bool cmd: default True プログレスバーを標準出力するかどうかのフラグ
         :return: results
         :rtype: list
         """
         results = []
-
-        if cmd:
-            # tqdm用のドキュメントの合計数を取得
-            total_bulk_lists = sum(
-                (len(i[collection_name]) for i in insert_data for
-                 collection_name, bulk_list in i.items() if
-                 isinstance(bulk_list, list)))
-            if not total_bulk_lists:  # embの場合
-                total_bulk_lists = 1
-
-            for i in insert_data:
-                collection_bar = tqdm(i.keys(), desc='collections', position=0)
-                doc_bar = tqdm(total=total_bulk_lists, desc='documents',
-                               position=1)
-                for collection, bulk_list in i.items():
-
-                    # データによっては
-                    # 単一のドキュメントの時にリストで囲まれていない場合がある
-                    if isinstance(bulk_list, dict):
-                        bulk_list = [bulk_list]
-
-                    try:
-                        result = self.db[collection].insert_many(bulk_list)
-                    except errors.BulkWriteError as e:
-                        raise EdmanDbProcessError(
-                            f'インサートに失敗しました:{e.details}\nインサート結果:{results}')
-
-                    results.append({collection: result.inserted_ids})
-                    # プログレスバー表示関係
-                    doc_bar.update(len(bulk_list))
-                    collection_bar.set_description(f'Processing {collection}')
-                    collection_bar.update(1)
-
-                doc_bar.close()
-                collection_bar.close()
-        else:
-            for i in insert_data:
-                for collection, bulk_list in i.items():
-                    if isinstance(bulk_list, dict):
-                        bulk_list = [bulk_list]
-                    try:
-                        result = self.db[collection].insert_many(bulk_list)
-                    except errors.BulkWriteError as e:
-                        raise EdmanDbProcessError(
-                            f'インサートに失敗しました:{e.details}\nインサート結果:{results}')
-                    results.append({collection: result.inserted_ids})
+        for i in insert_data:
+            for collection, bulk_list in i.items():
+                if isinstance(bulk_list, dict):
+                    bulk_list = [bulk_list]
+                try:
+                    result = self.db[collection].insert_many(bulk_list)
+                except errors.BulkWriteError as e:
+                    raise EdmanDbProcessError(
+                        f'インサートに失敗しました:{e.details}\nインサート結果:{results}')
+                results.append({collection: result.inserted_ids})
 
         return results
 
@@ -184,25 +348,25 @@ class DB:
 
         oid = Utils.conv_objectid(oid)
         doc = self.db[collection].find_one({'_id': oid})
+
         if doc is None:
-            # TODO resultはNoneのまま返却するように設計変更する予定
-            sys.exit('ドキュメントが存在しません')
+            result = None
+        else:
+            # embの場合は指定階層のドキュメントを引き抜く
+            # refの場合はdocの結果をそのまま入れる
+            try:
+                doc_result = self._get_emb_doc(doc,
+                                               query) if query is not None else doc
+            except EdmanInternalError:
+                raise
 
-        # embの場合は指定階層のドキュメントを引き抜く
-        # refの場合はdocの結果をそのまま入れる
-        try:
-            doc_result = self._get_emb_doc(doc,
-                                           query) if query is not None else doc
-        except EdmanInternalError:
-            raise
+            # クエリの指定によってはリストデータなども取得出てしまうため
+            if not isinstance(doc_result, dict):
+                raise EdmanInternalError(f'指定されたクエリはドキュメントではありません {query}')
 
-        # クエリの指定によってはリストデータなども取得出てしまうため
-        if not isinstance(doc_result, dict):
-            raise EdmanInternalError(f'指定されたクエリはドキュメントではありません {query}')
-
-        result = Utils.reference_item_delete(
-            doc_result, ('_id', self.parent, self.child, self.file_ref)
-        ) if reference_delete else doc_result
+            result = Utils.item_delete(
+                doc_result, ('_id', self.parent, self.child, self.file_ref)
+            ) if reference_delete else doc_result
 
         return result
 
@@ -216,18 +380,10 @@ class DB:
         :return: result
         :rtype: dict
         """
-        s = ''
-
-        for idx, i in enumerate(query):
-            if i.isdecimal():
-                s += '[' + i + ']'
-            else:
-                if idx != 0:
-                    s += '.'
-                s += i
+        s = Utils.generate_jms_query(query)
         try:
-            result = jmespath.search(s, doc)
-        except jmespath.parser.exceptions.ParseError:
+            result = jms_search(s, doc)
+        except jms_exceptions.ParseError:
             raise EdmanInternalError(f'クエリの変換が出来ませんでした: {s}')
 
         return result
@@ -255,7 +411,7 @@ class DB:
         if query is not None:  # emb
             try:
                 doc = Utils.doc_traverse(doc, [delete_key], query,
-                                         self._delete_execute)
+                                         Utils.item_delete)
             except Exception:
                 raise
         else:  # ref
@@ -270,20 +426,6 @@ class DB:
 
         return result
 
-    @staticmethod
-    def _delete_execute(doc: dict, keys: list):
-        """
-        ドキュメントの削除処理
-        _doc_traverse()のコールバック関数
-
-        :param dict doc:
-        :param list keys:
-        :return:
-        """
-        for key in keys:
-            if key in doc:
-                del doc[key]
-
     def _convert_datetime_dict(self, amend: dict) -> dict:
         """
         辞書内辞書になっている文字列日付時間データを、辞書内日付時間に変換
@@ -313,7 +455,8 @@ class DB:
                         buff = value
                     result.update({key: buff})
             except AttributeError:
-                raise EdmanInternalError(f'日付変換に失敗しました.構造に問題があります. {amend}')
+                raise EdmanInternalError(
+                    f'日付変換に失敗しました.構造に問題があります. {amend}')
         return result
 
     def update(self, collection: str, oid: Union[str, ObjectId],
@@ -333,8 +476,7 @@ class DB:
         oid = Utils.conv_objectid(oid)
         db_result = self.db[collection].find_one({'_id': oid})
         if db_result is None:
-            # TODO resultはNoneのまま返却するように設計変更する予定
-            sys.exit('該当するドキュメントは存在しません')
+            raise EdmanInternalError('該当するドキュメントは存在しません')
 
         if structure == 'emb':
             convert = Convert()
@@ -418,8 +560,7 @@ class DB:
         oid = Utils.conv_objectid(oid)
         db_result = self.db[collection].find_one({'_id': oid})
         if db_result is None:
-            # TODO resultはNoneのまま返却するように設計変更する予定
-            sys.exit('該当するドキュメントは存在しません')
+            raise EdmanInternalError('該当するドキュメントは存在しません')
 
         if structure == 'emb':
             try:
@@ -517,7 +658,7 @@ class DB:
                 break
         if target is None:
             raise ValueError(
-                '親となる' + parent_doc['id'] + 'に' + str(del_oid) + 'が登録されていません')
+                f'親となる{parent_doc["id"]}に{str(del_oid)}が登録されていません')
         else:
             children.remove(target)
 
@@ -533,7 +674,7 @@ class DB:
                     {'_id': ref.id}, parent_doc)
 
         if not result.modified_count:
-            raise ValueError('親となる' + parent_doc['id'] + 'は変更できませんでした')
+            raise ValueError(f'親となる{parent_doc["id"]}は変更できませんでした')
 
     def _extract_elements_from_doc(self, doc: dict, collection: str) -> dict:
         """
@@ -622,8 +763,7 @@ class DB:
         """
         doc = self.db[collection].find_one({'_id': Utils.conv_objectid(oid)})
         if doc is None:
-            # TODO resultはNoneのまま返却するように設計変更する予定
-            sys.exit('指定のドキュメントがありません')
+            raise EdmanInternalError('該当するドキュメントは存在しません')
 
         if any(key in doc for key in (self.parent, self.child)):
             return 'ref'
@@ -650,6 +790,9 @@ class DB:
             # 自分データ取り出し
             ref_result = self.doc(collection, oid, query=None,
                                   reference_delete=False)
+            if ref_result is None:
+                raise EdmanInternalError("ドキュメントが存在しません")
+
             reference_point_result = self.get_reference_point(
                 ref_result)
             if reference_point_result[self.child]:
@@ -710,10 +853,8 @@ class DB:
         def recursive(doc_list):
             # ここでデータを取得する
             for doc in doc_list:
-                tmp = self._child_storaged(doc)
-                if tmp:
+                if tmp := self._child_storaged(doc):
                     result.append(tmp)
-
                 # 子データがある時は繰り返す
                 if tmp:
                     recursive(tmp)
@@ -743,8 +884,7 @@ class DB:
                 tmp = []
                 # ここでデータを取得する
                 for doc in doc_list:
-                    tmp = self._child_storaged(doc)
-                    if tmp:
+                    if tmp := self._child_storaged(doc):
                         result.append(tmp)
                 depth -= 1
 
@@ -811,15 +951,15 @@ class DB:
         :return:
         :rtype: dict
         """
-        result = {}
-        for bros in find_result:
+
+        def foo(b):
             try:
-                parent_id = self._get_uni_parent(bros)
+                parent_id = self._get_uni_parent(b)
             except ValueError:
                 raise
-            result.update({parent_id: bros})
+            return parent_id
 
-        return result
+        return {foo(bros): bros for bros in find_result}
 
     def _get_uni_parent(self, bros: dict) -> ObjectId:
         """
@@ -894,18 +1034,12 @@ class DB:
         :return: result
         :rtype: dict
         """
-        docs = self.db[collection].find()
-        if docs.count() == 0:
-            # TODO resultはNoneのまま返却するように設計変更する予定
-            sys.exit('対象のドキュメントは存在しません')
-        id_list = []
-        for doc in docs:
-            if self.get_structure(collection, doc['_id']) == 'emb':
-                id_list.append(doc['_id'])
 
-        if len(id_list) == 0:
-            # TODO これは正常終了させても問題なさそう
-            sys.exit('変換対象のドキュメントは全てreferenceです')
+        if self.db[collection].estimated_document_count() == 0:
+            raise EdmanInternalError('該当するドキュメントは存在しません')
+
+        id_list = [doc['_id'] for doc in self.db[collection].find() if
+                   self.get_structure(collection, doc['_id']) == 'emb']
 
         result_list = []
         for oid in id_list:
@@ -931,7 +1065,6 @@ class DB:
         result = {}
         if len(result_list):
             result.update({'result': result_list})
-            print('\r\n')  # 改行できない問題を回避
 
         return result
 
@@ -983,32 +1116,32 @@ class DB:
         DB内のデータをJSONに従って型変更をする
         DBにあってJSONにないキーは無視
         スペルミスも含め型一覧にない型を指定した時はstrに変換
+        |
+        |   JSON例:
+        |   {
+        |       "コレクション名":{
+        |           "キー": "変更する型",
+        |           "キー2": "変更する型",
+        |       },
+        |       "コレクション名2":{
+        |           "キー": ["変更する型","変更する型"],
+        |       }
+        |   }
 
-        JSON例:
-        {
-            "コレクション名":{
-                "キー": "変更する型",
-                "キー2": "変更する型",
-            },
-            "コレクション名2":{
-                "キー": ["変更する型","変更する型"],
-            }
-        }
+        |   値がリストの時
+        |       ・双方どちらかがリストでない時は無視
+        |       ・JSON側が単一、DB側が複数の時は単一の型で全て変換する
+        |           JSON:['str']
+        |           DB:['1','2','3']
+        |       ・JSON側よりDB側が少ない時はJSON側は切り捨て
+        |           JSON:['str'、'int', 'int']
+        |           DB:['1',2]
+        |       ・DB側よりJSON側が少ない時は、リストの最後の型で繰り返す
+        |           JSON:['str'、'int']
+        |           DB:['1',2,3,4,5]
 
-        値がリストの時
-            ・双方どちらかがリストでない時は無視
-            ・JSON側が単一、DB側が複数の時は単一の型で全て変換する
-                JSON:['str']
-                DB:['1','2','3']
-            ・JSON側よりDB側が少ない時はJSON側は切り捨て
-                JSON:['str'、'int', 'int']
-                DB:['1',2]
-            ・DB側よりJSON側が少ない時は、リストの最後の型で繰り返す
-                JSON:['str'、'int']
-                DB:['1',2,3,4,5]
-
-        型一覧:
-        [int,float,bool,str,datetime]
+        |   型一覧:
+        |   [int,float,bool,str,datetime]
 
         :param dict bson_data:
         :return: result
@@ -1021,14 +1154,14 @@ class DB:
             # フィルタ、プロジェクション作成
             items_keys = list(items.keys())
             projection = {k: 1 for k in items_keys}
-            filter = {'$or': [
+            reference = {'$or': [
                 {
                     self.child: {'$exists': True}
                 },
                 {
                     self.parent: {'$exists': True}
                 }]}
-            docs = self.db[collection].find(filter, projection=projection)
+            docs = self.db[collection].find(reference, projection=projection)
 
             for doc in docs:
                 update_params = {}
@@ -1051,19 +1184,11 @@ class DB:
                                     list_buff.append(f(list_value))
                                 param = {item_key: list_buff}
                             else:
-                                # DB側がリストじゃない時
-                                # TODO warningで出したほうが良さそう
-                                print(
-                                    f'DB側はリストではありません.無視します '
-                                    f'{collection} {doc["_id"]} {item_key}')
+                                # DB側がリストじゃない時は無視
                                 param = {}
                         else:
-                            # JSONがリストじゃないのにDBがリストの時
+                            # JSONがリストじゃないのにDBがリストの時は無視
                             if isinstance(db_value, list):
-                                # TODO warningで出したほうが良さそう
-                                print(
-                                    f'DB側はリストですがJSONでリストが指定されていません.無視します '
-                                    f'{collection} {doc["_id"]} {item_key}')
                                 param = {}
                             else:
                                 # DBもJSONも単一の型の時
